@@ -31,6 +31,9 @@
 #define TYPE_STOPITER           'S'
 #define TYPE_ELLIPSIS           '.'
 #define TYPE_INT                'i'
+/* TYPE_INT64 is deprecated. It is not
+   generated anymore, and support for reading it
+   will be removed in Python 3.4. */
 #define TYPE_INT64              'I'
 #define TYPE_FLOAT              'f'
 #define TYPE_BINARY_FLOAT       'g'
@@ -59,9 +62,9 @@ typedef struct {
     /* If fp == NULL, the following are valid: */
     PyObject *readable;    /* Stream-like object being read from */
     PyObject *str;
+    PyObject *current_filename;
     char *ptr;
     char *end;
-    PyObject *strings; /* dict on marshal, list on unmarshal */
     int version;
 } WFILE;
 
@@ -92,7 +95,7 @@ w_more(int c, WFILE *p)
 }
 
 static void
-w_string(char *s, Py_ssize_t n, WFILE *p)
+w_string(char *s, int n, WFILE *p)
 {
     if (p->fp != NULL) {
         fwrite(s, 1, n, p->fp);
@@ -121,30 +124,6 @@ w_long(long x, WFILE *p)
     w_byte((char)((x>>24) & 0xff), p);
 }
 
-#if SIZEOF_LONG > 4
-static void
-w_long64(long x, WFILE *p)
-{
-    w_long(x, p);
-    w_long(x>>32, p);
-}
-#endif
-
-#define SIZE32_MAX  0x7FFFFFFF
-
-#if SIZEOF_SIZE_T > 4
-# define W_SIZE(n, p)  do {                     \
-        if ((n) > SIZE32_MAX) {                 \
-            (p)->depth--;                       \
-            (p)->error = WFERR_UNMARSHALLABLE;  \
-            return;                             \
-        }                                       \
-        w_long((long)(n), p);                   \
-    } while(0)
-#else
-# define W_SIZE  w_long
-#endif
-
 /* We assume that Python longs are stored internally in base some power of
    2**15; for the sake of portability we'll always read and write them in base
    exactly 2**15. */
@@ -178,11 +157,6 @@ w_PyLong(const PyLongObject *ob, WFILE *p)
         d >>= PyLong_MARSHAL_SHIFT;
         l++;
     } while (d != 0);
-    if (l > SIZE32_MAX) {
-        p->depth--;
-        p->error = WFERR_UNMARSHALLABLE;
-        return;
-    }
     w_long((long)(Py_SIZE(ob) > 0 ? l : -l), p);
 
     for (i=0; i < n-1; i++) {
@@ -239,8 +213,8 @@ w_object(PyObject *v, WFILE *p)
 #if SIZEOF_LONG > 4
             long y = Py_ARITHMETIC_RIGHT_SHIFT(long, x, 31);
             if (y && y != -1) {
-                w_byte(TYPE_INT64, p);
-                w_long64(x, p);
+                /* Too large for TYPE_INT */
+                w_PyLong((PyLongObject*)v, p);
             }
             else
 #endif
@@ -271,7 +245,7 @@ w_object(PyObject *v, WFILE *p)
             n = strlen(buf);
             w_byte(TYPE_FLOAT, p);
             w_byte((int)n, p);
-            w_string(buf, n, p);
+            w_string(buf, (int)n, p);
             PyMem_Free(buf);
         }
     }
@@ -303,7 +277,7 @@ w_object(PyObject *v, WFILE *p)
             }
             n = strlen(buf);
             w_byte((int)n, p);
-            w_string(buf, n, p);
+            w_string(buf, (int)n, p);
             PyMem_Free(buf);
             buf = PyOS_double_to_string(PyComplex_ImagAsDouble(v),
                                         'g', 17, 0, NULL);
@@ -313,21 +287,25 @@ w_object(PyObject *v, WFILE *p)
             }
             n = strlen(buf);
             w_byte((int)n, p);
-            w_string(buf, n, p);
+            w_string(buf, (int)n, p);
             PyMem_Free(buf);
         }
     }
     else if (PyBytes_CheckExact(v)) {
         w_byte(TYPE_STRING, p);
         n = PyBytes_GET_SIZE(v);
-        W_SIZE(n, p);
-        w_string(PyBytes_AS_STRING(v), n, p);
+        if (n > INT_MAX) {
+            /* huge strings are not supported */
+            p->depth--;
+            p->error = WFERR_UNMARSHALLABLE;
+            return;
+        }
+        w_long((long)n, p);
+        w_string(PyBytes_AS_STRING(v), (int)n, p);
     }
     else if (PyUnicode_CheckExact(v)) {
         PyObject *utf8;
-        utf8 = PyUnicode_EncodeUTF8(PyUnicode_AS_UNICODE(v),
-                                    PyUnicode_GET_SIZE(v),
-                                    "surrogatepass");
+        utf8 = PyUnicode_AsEncodedString(v, "utf8", "surrogatepass");
         if (utf8 == NULL) {
             p->depth--;
             p->error = WFERR_UNMARSHALLABLE;
@@ -335,14 +313,19 @@ w_object(PyObject *v, WFILE *p)
         }
         w_byte(TYPE_UNICODE, p);
         n = PyBytes_GET_SIZE(utf8);
-        W_SIZE(n, p);
-        w_string(PyBytes_AS_STRING(utf8), n, p);
+        if (n > INT_MAX) {
+            p->depth--;
+            p->error = WFERR_UNMARSHALLABLE;
+            return;
+        }
+        w_long((long)n, p);
+        w_string(PyBytes_AS_STRING(utf8), (int)n, p);
         Py_DECREF(utf8);
     }
     else if (PyTuple_CheckExact(v)) {
         w_byte(TYPE_TUPLE, p);
         n = PyTuple_Size(v);
-        W_SIZE(n, p);
+        w_long((long)n, p);
         for (i = 0; i < n; i++) {
             w_object(PyTuple_GET_ITEM(v, i), p);
         }
@@ -350,7 +333,7 @@ w_object(PyObject *v, WFILE *p)
     else if (PyList_CheckExact(v)) {
         w_byte(TYPE_LIST, p);
         n = PyList_GET_SIZE(v);
-        W_SIZE(n, p);
+        w_long((long)n, p);
         for (i = 0; i < n; i++) {
             w_object(PyList_GET_ITEM(v, i), p);
         }
@@ -380,7 +363,7 @@ w_object(PyObject *v, WFILE *p)
             p->error = WFERR_UNMARSHALLABLE;
             return;
         }
-        W_SIZE(n, p);
+        w_long((long)n, p);
         it = PyObject_GetIter(v);
         if (it == NULL) {
             p->depth--;
@@ -430,8 +413,13 @@ w_object(PyObject *v, WFILE *p)
         w_byte(TYPE_STRING, p);
         n = view.len;
         s = view.buf;
-        W_SIZE(n, p);
-        w_string(s, n, p);
+        if (n > INT_MAX) {
+            p->depth--;
+            p->error = WFERR_UNMARSHALLABLE;
+            return;
+        }
+        w_long((long)n, p);
+        w_string(s, (int)n, p);
         PyBuffer_Release(&view);
     }
     else {
@@ -449,7 +437,6 @@ PyMarshal_WriteLongToFile(long x, FILE *fp, int version)
     wf.fp = fp;
     wf.error = WFERR_OK;
     wf.depth = 0;
-    wf.strings = NULL;
     wf.version = version;
     w_long(x, &wf);
 }
@@ -461,35 +448,35 @@ PyMarshal_WriteObjectToFile(PyObject *x, FILE *fp, int version)
     wf.fp = fp;
     wf.error = WFERR_OK;
     wf.depth = 0;
-    wf.strings = (version > 0) ? PyDict_New() : NULL;
     wf.version = version;
     w_object(x, &wf);
-    Py_XDECREF(wf.strings);
 }
 
 typedef WFILE RFILE; /* Same struct with different invariants */
 
 #define rs_byte(p) (((p)->ptr < (p)->end) ? (unsigned char)*(p)->ptr++ : EOF)
 
-static Py_ssize_t
-r_string(char *s, Py_ssize_t n, RFILE *p)
+static int
+r_string(char *s, int n, RFILE *p)
 {
     char *ptr;
-    Py_ssize_t read, left;
+    int read, left;
 
     if (!p->readable) {
         if (p->fp != NULL)
             /* The result fits into int because it must be <=n. */
-            read = fread(s, 1, n, p->fp);
+            read = (int) fread(s, 1, n, p->fp);
         else {
-            left = p->end - p->ptr;
+            left = (int)(p->end - p->ptr);
             read = (left < n) ? left : n;
             memcpy(s, p->ptr, read);
             p->ptr += read;
         }
     }
     else {
-        PyObject *data = PyObject_CallMethod(p->readable, "read", "n", n);
+        _Py_IDENTIFIER(read);
+
+        PyObject *data = _PyObject_CallMethodId(p->readable, &PyId_read, "i", n);
         read = 0;
         if (data != NULL) {
             if (!PyBytes_Check(data)) {
@@ -519,7 +506,7 @@ r_byte(RFILE *p)
 {
     int c = EOF;
     unsigned char ch;
-    Py_ssize_t n;
+    int n;
 
     if (!p->readable)
         c = p->fp ? getc(p->fp) : rs_byte(p);
@@ -603,8 +590,8 @@ static PyObject *
 r_PyLong(RFILE *p)
 {
     PyLongObject *ob;
-    long n, size, i;
-    int j, md, shorts_in_top_digit;
+    int size, i, j, md, shorts_in_top_digit;
+    long n;
     digit d;
 
     n = r_long(p);
@@ -612,7 +599,7 @@ r_PyLong(RFILE *p)
         return NULL;
     if (n == 0)
         return (PyObject *)_PyLong_New(0);
-    if (n < -SIZE32_MAX || n > SIZE32_MAX) {
+    if (n < -INT_MAX || n > INT_MAX) {
         PyErr_SetString(PyExc_ValueError,
                        "bad marshal data (long size out of range)");
         return NULL;
@@ -743,7 +730,7 @@ r_object(RFILE *p)
             double dx;
             retval = NULL;
             n = r_byte(p);
-            if (n == EOF || r_string(buf, n, p) != n) {
+            if (n == EOF || r_string(buf, (int)n, p) != n) {
                 PyErr_SetString(PyExc_EOFError,
                     "EOF read where object expected");
                 break;
@@ -781,7 +768,7 @@ r_object(RFILE *p)
             Py_complex c;
             retval = NULL;
             n = r_byte(p);
-            if (n == EOF || r_string(buf, n, p) != n) {
+            if (n == EOF || r_string(buf, (int)n, p) != n) {
                 PyErr_SetString(PyExc_EOFError,
                     "EOF read where object expected");
                 break;
@@ -791,7 +778,7 @@ r_object(RFILE *p)
             if (c.real == -1.0 && PyErr_Occurred())
                 break;
             n = r_byte(p);
-            if (n == EOF || r_string(buf, n, p) != n) {
+            if (n == EOF || r_string(buf, (int)n, p) != n) {
                 PyErr_SetString(PyExc_EOFError,
                     "EOF read where object expected");
                 break;
@@ -840,7 +827,7 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (n < 0 || n > SIZE32_MAX) {
+        if (n < 0 || n > INT_MAX) {
             PyErr_SetString(PyExc_ValueError, "bad marshal data (string size out of range)");
             retval = NULL;
             break;
@@ -850,7 +837,7 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (r_string(PyBytes_AS_STRING(v), n, p) != n) {
+        if (r_string(PyBytes_AS_STRING(v), (int)n, p) != n) {
             Py_DECREF(v);
             PyErr_SetString(PyExc_EOFError,
                             "EOF read where object expected");
@@ -869,7 +856,7 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (n < 0 || n > SIZE32_MAX) {
+        if (n < 0 || n > INT_MAX) {
             PyErr_SetString(PyExc_ValueError, "bad marshal data (unicode size out of range)");
             retval = NULL;
             break;
@@ -879,7 +866,7 @@ r_object(RFILE *p)
             retval = PyErr_NoMemory();
             break;
         }
-        if (r_string(buffer, n, p) != n) {
+        if (r_string(buffer, (int)n, p) != n) {
             PyMem_DEL(buffer);
             PyErr_SetString(PyExc_EOFError,
                 "EOF read where object expected");
@@ -898,12 +885,12 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (n < 0 || n > SIZE32_MAX) {
+        if (n < 0 || n > INT_MAX) {
             PyErr_SetString(PyExc_ValueError, "bad marshal data (tuple size out of range)");
             retval = NULL;
             break;
         }
-        v = PyTuple_New(n);
+        v = PyTuple_New((int)n);
         if (v == NULL) {
             retval = NULL;
             break;
@@ -918,7 +905,7 @@ r_object(RFILE *p)
                 v = NULL;
                 break;
             }
-            PyTuple_SET_ITEM(v, i, v2);
+            PyTuple_SET_ITEM(v, (int)i, v2);
         }
         retval = v;
         break;
@@ -929,12 +916,12 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (n < 0 || n > SIZE32_MAX) {
+        if (n < 0 || n > INT_MAX) {
             PyErr_SetString(PyExc_ValueError, "bad marshal data (list size out of range)");
             retval = NULL;
             break;
         }
-        v = PyList_New(n);
+        v = PyList_New((int)n);
         if (v == NULL) {
             retval = NULL;
             break;
@@ -949,7 +936,7 @@ r_object(RFILE *p)
                 v = NULL;
                 break;
             }
-            PyList_SET_ITEM(v, i, v2);
+            PyList_SET_ITEM(v, (int)i, v2);
         }
         retval = v;
         break;
@@ -985,7 +972,7 @@ r_object(RFILE *p)
             retval = NULL;
             break;
         }
-        if (n < 0 || n > SIZE32_MAX) {
+        if (n < 0 || n > INT_MAX) {
             PyErr_SetString(PyExc_ValueError, "bad marshal data (set size out of range)");
             retval = NULL;
             break;
@@ -1073,6 +1060,18 @@ r_object(RFILE *p)
             filename = r_object(p);
             if (filename == NULL)
                 goto code_error;
+            if (PyUnicode_CheckExact(filename)) {
+                if (p->current_filename != NULL) {
+                    if (!PyUnicode_Compare(filename, p->current_filename)) {
+                        Py_DECREF(filename);
+                        Py_INCREF(p->current_filename);
+                        filename = p->current_filename;
+                    }
+                }
+                else {
+                    p->current_filename = filename;
+                }
+            }
             name = r_object(p);
             if (name == NULL)
                 goto code_error;
@@ -1135,7 +1134,7 @@ PyMarshal_ReadShortFromFile(FILE *fp)
     assert(fp);
     rf.readable = NULL;
     rf.fp = fp;
-    rf.strings = NULL;
+    rf.current_filename = NULL;
     rf.end = rf.ptr = NULL;
     return r_short(&rf);
 }
@@ -1146,7 +1145,7 @@ PyMarshal_ReadLongFromFile(FILE *fp)
     RFILE rf;
     rf.fp = fp;
     rf.readable = NULL;
-    rf.strings = NULL;
+    rf.current_filename = NULL;
     rf.ptr = rf.end = NULL;
     return r_long(&rf);
 }
@@ -1181,8 +1180,12 @@ PyMarshal_ReadLastObjectFromFile(FILE *fp)
     if (filesize > 0 && filesize <= REASONABLE_FILE_LIMIT) {
         char* pBuf = (char *)PyMem_MALLOC(filesize);
         if (pBuf != NULL) {
-            size_t n = fread(pBuf, 1, (size_t)filesize, fp);
-            PyObject* v = PyMarshal_ReadObjectFromString(pBuf, n);
+            PyObject* v;
+            size_t n;
+            /* filesize must fit into an int, because it
+               is smaller than REASONABLE_FILE_LIMIT */
+            n = fread(pBuf, 1, (int)filesize, fp);
+            v = PyMarshal_ReadObjectFromString(pBuf, n);
             PyMem_FREE(pBuf);
             return v;
         }
@@ -1204,11 +1207,10 @@ PyMarshal_ReadObjectFromFile(FILE *fp)
     PyObject *result;
     rf.fp = fp;
     rf.readable = NULL;
-    rf.strings = PyList_New(0);
+    rf.current_filename = NULL;
     rf.depth = 0;
     rf.ptr = rf.end = NULL;
     result = r_object(&rf);
-    Py_DECREF(rf.strings);
     return result;
 }
 
@@ -1219,12 +1221,11 @@ PyMarshal_ReadObjectFromString(char *str, Py_ssize_t len)
     PyObject *result;
     rf.fp = NULL;
     rf.readable = NULL;
+    rf.current_filename = NULL;
     rf.ptr = str;
     rf.end = str + len;
-    rf.strings = PyList_New(0);
     rf.depth = 0;
     result = r_object(&rf);
-    Py_DECREF(rf.strings);
     return result;
 }
 
@@ -1232,7 +1233,6 @@ PyObject *
 PyMarshal_WriteObjectToString(PyObject *x, int version)
 {
     WFILE wf;
-    PyObject *res = NULL;
 
     wf.fp = NULL;
     wf.readable = NULL;
@@ -1244,9 +1244,7 @@ PyMarshal_WriteObjectToString(PyObject *x, int version)
     wf.error = WFERR_OK;
     wf.depth = 0;
     wf.version = version;
-    wf.strings = (version > 0) ? PyDict_New() : NULL;
     w_object(x, &wf);
-    Py_XDECREF(wf.strings);
     if (wf.str != NULL) {
         char *base = PyBytes_AS_STRING((PyBytesObject *)wf.str);
         if (wf.ptr - base > PY_SSIZE_T_MAX) {
@@ -1268,12 +1266,7 @@ PyMarshal_WriteObjectToString(PyObject *x, int version)
                :"object too deeply nested to marshal");
         return NULL;
     }
-    if (wf.str != NULL) {
-        /* XXX Quick hack -- need to do this differently */
-        res = PyBytes_FromObject(wf.str);
-        Py_DECREF(wf.str);
-    }
-    return res;
+    return wf.str;
 }
 
 /* And an interface for Python programs... */
@@ -1287,12 +1280,14 @@ marshal_dump(PyObject *self, PyObject *args)
     int version = Py_MARSHAL_VERSION;
     PyObject *s;
     PyObject *res;
+    _Py_IDENTIFIER(write);
+
     if (!PyArg_ParseTuple(args, "OO|i:dump", &x, &f, &version))
         return NULL;
     s = PyMarshal_WriteObjectToString(x, version);
     if (s == NULL)
         return NULL;
-    res = PyObject_CallMethod(f, "write", "O", s);
+    res = _PyObject_CallMethodId(f, &PyId_write, "O", s);
     Py_DECREF(s);
     return res;
 }
@@ -1314,6 +1309,7 @@ static PyObject *
 marshal_load(PyObject *self, PyObject *f)
 {
     PyObject *data, *result;
+    _Py_IDENTIFIER(read);
     RFILE rf;
 
     /*
@@ -1321,7 +1317,7 @@ marshal_load(PyObject *self, PyObject *f)
      * This is to ensure that the object passed in at least
      * has a read method which returns bytes.
      */
-    data = PyObject_CallMethod(f, "read", "i", 0);
+    data = _PyObject_CallMethodId(f, &PyId_read, "i", 0);
     if (data == NULL)
         return NULL;
     if (!PyBytes_Check(data)) {
@@ -1331,12 +1327,11 @@ marshal_load(PyObject *self, PyObject *f)
         result = NULL;
     }
     else {
-        rf.strings = PyList_New(0);
         rf.depth = 0;
         rf.fp = NULL;
         rf.readable = f;
+        rf.current_filename = NULL;
         result = read_object(&rf);
-        Py_DECREF(rf.strings);
     }
     Py_DECREF(data);
     return result;
@@ -1389,12 +1384,11 @@ marshal_loads(PyObject *self, PyObject *args)
     n = p.len;
     rf.fp = NULL;
     rf.readable = NULL;
+    rf.current_filename = NULL;
     rf.ptr = s;
     rf.end = s + n;
-    rf.strings = PyList_New(0);
     rf.depth = 0;
     result = read_object(&rf);
-    Py_DECREF(rf.strings);
     PyBuffer_Release(&p);
     return result;
 }
